import { Store } from "@tsed/core";
/**
 * Attach the decorated method to the socket event.
 *
 * ### Example
 *
 * ```typescript
 * @SocketService("/nsp")
 * export class MyWS {
 *
 *   @Input("event")
 *   myMethod(@Args(0) data: any, @Nsp socket) {
 *
 *   }
 * }
 * ```
 *
 * @decorator
 * @param eventName
 */
export function Input(eventName) {
    return (target, propertyKey, descriptor) => {
        Store.from(target).merge("socketIO", {
            handlers: {
                [propertyKey]: {
                    eventName,
                    methodClassName: propertyKey
                }
            }
        });
    };
}
