import { SocketFilters } from "../interfaces/SocketFilters.js";
import { SocketFilter } from "./socketFilter.js";
/**
 * Inject the [SocketIO.Namespace](https://socket.io/docs/rooms-and-namespaces/#namespaces) instance in the decorated parameter.
 * This namespace may differ from the default namespace when using dynamic namespaces.
 *
 * ### Example
 *
 * ```typescript
 * @SocketService(/nsp-.+/)
 * export class MyWS {
 *
 *   @Input("event")
 *   myMethod(@SocketNsp nsp: SocketNsp) {
 *
 *   }
 * }
 * ```
 *
 * @experimental
 * @param target
 * @param {string} propertyKey
 * @param {number} index
 * @decorator
 */
export function SocketNsp(target, propertyKey, index) {
    return SocketFilter(SocketFilters.SOCKET_NSP)(target, propertyKey, index);
}
