import { SocketFilters } from "../interfaces/SocketFilters.js";
import { SocketFilter } from "./socketFilter.js";
/**
 * Inject the socket session data in the decorated parameter.
 *
 * ### Example
 *
 * ```typescript
 * @SocketService("/nsp")
 * export class MyWS {
 *
 *   @Input("event")
 *   myMethod(@SocketSession session: SocketSession) {
 *     console.log(session);
 *   }
 * }
 * ```
 *
 * @param target
 * @param {string} propertyKey
 * @param {number} index
 * @decorator
 */
export function SocketSession(target, propertyKey, index) {
    return SocketFilter(SocketFilters.SESSION)(target, propertyKey, index);
}
/**
 * Inject the raw `socket.data` in the decorated parameter.
 *
 * @example
 * ```typescript
 * @SocketService("/nsp")
 * export class MyWS {
 *
 *   @Input("event")
 *   myMethod(@RawSocketSession session: Record<string, unknown>) {
 *     console.log(session);
 *   }
 * }
 * ```
 *
 * @param target
 * @param {string} propertyKey
 * @param {number} index
 * @constructor
 */
export function RawSocketSession(target, propertyKey, index) {
    return SocketFilter(SocketFilters.RAW_SESSION)(target, propertyKey, index);
}
